#region usings
using System;
using System.ComponentModel.Composition;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Drawing;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
using System.ComponentModel;
#endregion usings

namespace VVVV.Nodes.TableBuffer
{
	#region PluginInfo
	[PluginInfo(Name = "TableViewString", Category = "SpreadTable", Help = "Use a .NET DataGridView to view a SpreadTable", Tags = "", AutoEvaluate = true)]
	#endregion PluginInfo
	public class SpreadTableViewNode : UserControl, IPluginEvaluate
	{
		#region fields & pins
		[Input("SetView", IsSingle = true)]
		IDiffSpread<int> FSetIndex;
		
		[Input("Go", IsSingle = true)]
		ISpread<bool> FGo;
		
		[Input("SetTimes", IsSingle = true)]
		ISpread<bool> FSetTimes;
		
		[Input("Edit", IsSingle = true)]
		ISpread<bool> FEdit;
		
		[Input("CueNumber", DefaultString = "keineZeit")]
		ISpread<string> FCueNumbers;
		
		[Input("TimeCodes", DefaultString = "keineZeit")]
		ISpread<string> FTimeCodes;

		[Input("Table", IsSingle = true)]
		ISpread<SpreadTable> FPinInTable;

		[Output("Index")]
		ISpread<int> FCurrentIndex;
		
		[Output("RowCount")]
		ISpread<int> FRowCount;
		
		[Output("EndEdit")]
		ISpread<int> FEndEdit;
		
		

		[Import()]
		ILogger FLogger;

		DataGridView FDataGridView;
		SpreadTable FData;
		bool FNeedsUpdate = false;
		#endregion fields & pins

		#region constructor and init

		public SpreadTableViewNode()
		{
			//setup the gui
			InitializeComponent();
		}

		void InitializeComponent()
		{
			System.Windows.Forms.DataGridViewCellStyle dataGridViewCellStyle1 = new System.Windows.Forms.DataGridViewCellStyle();
			System.Windows.Forms.DataGridViewCellStyle dataGridViewCellStyle2 = new System.Windows.Forms.DataGridViewCellStyle();
			this.FDataGridView = new System.Windows.Forms.DataGridView();
			((System.ComponentModel.ISupportInitialize)(this.FDataGridView)).BeginInit();
			this.SuspendLayout();
			// 
			// FDataGridView
			// 
			dataGridViewCellStyle1.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(224)))), ((int)(((byte)(224)))), ((int)(((byte)(224)))));
			//dataGridViewCellStyle1.Format = "N4";
			//dataGridViewCellStyle1.NullValue = "0.0000";
			dataGridViewCellStyle1.SelectionBackColor = System.Drawing.Color.Gray;
			this.FDataGridView.AlternatingRowsDefaultCellStyle = dataGridViewCellStyle1;
			//hier änderung, vorher Fill
			this.FDataGridView.AutoSizeColumnsMode = System.Windows.Forms.DataGridViewAutoSizeColumnsMode.ColumnHeader;
			this.FDataGridView.ColumnHeadersHeightSizeMode = System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
			this.FDataGridView.Cursor = System.Windows.Forms.Cursors.Default;
			this.FDataGridView.Location = new System.Drawing.Point(0, 0);
			this.FDataGridView.Name = "FDataGridView";
			//dataGridViewCellStyle2.Format = "N4";
			//dataGridViewCellStyle2.NullValue = "0.0000";
			dataGridViewCellStyle2.SelectionBackColor = System.Drawing.Color.Gray;
			this.FDataGridView.RowsDefaultCellStyle = dataGridViewCellStyle2;
			this.FDataGridView.Size = this.Size;
			
			this.FDataGridView.TabIndex = 0;
			this.FDataGridView.CellValidating += new System.Windows.Forms.DataGridViewCellValidatingEventHandler(this.dataGridView1_CellValidating);
			this.FDataGridView.MouseMove += new System.Windows.Forms.MouseEventHandler(FDataGridView_MouseMove);
			
			//Column.SortMode = DataGridViewColumnSortMode.NotSortable;
			// ValueTableBufferNode
			// 
			
			//this.FDataGridView.CellMouseClick += new System.Windows.Forms.DataGridViewCellMouseEventHandler(this.FDataGridView_CellMouseClick);
            this.FDataGridView.KeyDown += new System.Windows.Forms.KeyEventHandler(this.dataGridView1_KeyDown);
			
			
			this.Controls.Add(this.FDataGridView);
			this.Name = "ValueTableBufferNode";
			this.Size = new System.Drawing.Size(344, 368);
			this.Resize += new System.EventHandler(this.ValueTableBufferNode_Resize);
			((System.ComponentModel.ISupportInitialize)(this.FDataGridView)).EndInit();
			
			this.ResumeLayout(false);

		}

		Point FMouseLast;
		
		bool FMouseDragging = false;
		
		private void dataGridView1_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                if (e.Modifiers == Keys.Control)
                {
                    switch (e.KeyCode)
                    {
                        case Keys.C:
                    	FLogger.Log(LogType.Debug, "copy");
                    	
                            if (FEdit[0]==true) CopyToClipboard();
                            break;
                    	MessageBox.Show("Copy/paste operation failed. ");

                        case Keys.V:
                            if (FEdit[0]==true) PasteClipboardValue();
                   
                    	
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Copy/paste operation failed. "+ex.Message, "Copy/Paste", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            FLogger.Log(LogType.Debug, "failed");	
            }
        }
		
		
	private void CopyToClipboard()
        {
            //Copy to clipboard
            DataObject dataObj = FDataGridView.GetClipboardContent();
            if (dataObj != null)
                Clipboard.SetDataObject(dataObj);
        }

	private void PasteClipboardValue()
        {
            //Show Error if no cell is selected
            if (FDataGridView.SelectedCells.Count == 0)
            {
                MessageBox.Show("Please select a cell", "Paste", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            //Get the satring Cell
            DataGridViewCell startCell = GetStartCell(FDataGridView);
        	
            //Get the clipboard value in a dictionary
            Dictionary<int, Dictionary<int, string>> cbValue = ClipBoardValues(Clipboard.GetText());
			
            int iRowIndex = startCell.RowIndex;
            foreach (int rowKey in cbValue.Keys)
            {
                int iColIndex = startCell.ColumnIndex;
                foreach (int cellKey in cbValue[rowKey].Keys)
                {
                    //Check if the index is with in the limit
                    if (iColIndex <= FDataGridView.Columns.Count - 1 && iRowIndex <= FDataGridView.Rows.Count - 1)
                    {
                        DataGridViewCell cell = FDataGridView[iColIndex, iRowIndex];
						 
                        //Copy to selected cells if 'chkPasteToSelectedCells' is checked
						cell.Value = cbValue[rowKey][cellKey];
                    }
                    iColIndex++;
                }
                iRowIndex++;
            }
        }

        private DataGridViewCell GetStartCell(DataGridView dgView)
        {
            //get the smallest row,column index
            if (dgView.SelectedCells.Count == 0)
                return null;

            int rowIndex = dgView.Rows.Count - 1;
            int colIndex = dgView.Columns.Count - 1;

            foreach (DataGridViewCell dgvCell in dgView.SelectedCells)
            {
                if (dgvCell.RowIndex < rowIndex)
                    rowIndex = dgvCell.RowIndex;
                if (dgvCell.ColumnIndex < colIndex)
                    colIndex = dgvCell.ColumnIndex;
            }

            return dgView[colIndex, rowIndex];
        }

        private Dictionary<int, Dictionary<int, string>> ClipBoardValues(string clipboardValue)
        {
            Dictionary<int, Dictionary<int, string>> copyValues = new Dictionary<int, Dictionary<int, string>>();

            String[] lines = clipboardValue.Split('\n');

            for (int i = 0; i <= lines.Length - 1; i++)
            {
                copyValues[i] = new Dictionary<int, string>();
                String[] lineContent = lines[i].Split('\t');

                //if an empty cell value copied, then set the dictionay with an empty string
                //else Set value to dictionary
                if (lineContent.Length == 0)
                    copyValues[i][0] = string.Empty;
                else
                {
                    for (int j = 0; j <= lineContent.Length - 1; j++)
                        copyValues[i][j] = lineContent[j];
                }
            }
            return copyValues;
        }
		
		
		
		void FDataGridView_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button.HasFlag(System.Windows.Forms.MouseButtons.Right))
			{
				if (FMouseDragging)
				{
					double delta = - 0.01 * (double)(e.Y - FMouseLast.Y);
					foreach (DataGridViewCell cell in FDataGridView.SelectedCells)
						if (cell.Value.GetType() == typeof(System.Double) && cell.RowIndex < FData.Rows.Count) //avoids selection of the 'new row' at bottom or invalid cells
							cell.Value = (double)cell.Value + delta;
					FData.OnDataChange(this);
				}
				else
				{
					FMouseDragging = true;
				}
				FMouseLast = e.Location;
			}
			else
				FMouseDragging = false;
		}
		

		#endregion constructor and init

		public void Evaluate(int SpreadMax)
		{
			if (FPinInTable[0] != FData)
			{
				if (FData != null)
				{
					FData.DataChanged -= new SpreadTable.DataChangedHandler(FData_DataChanged);
				}
				FData = FPinInTable[0];
				if (FData != null)
				{
					FData.DataChanged += new SpreadTable.DataChangedHandler(FData_DataChanged);
				}
				FDataGridView.DataSource = FData;
			}

			if (FData == null)
				return;
			
			if (FEdit[0]==true)
			{//FDataGridView.AllowUserToAddRows = true;
				
			
			}
			else
			{//FDataGridView.AllowUserToAddRows = false;
				FDataGridView.CancelEdit();
			}
			
			
			
			if (FData.Rows.Count > 0)
			{
				bool moveRow = false;
				int selectedRow = 0;
				FRowCount[0] = FData.Rows.Count;
				
				
				
				// hier kommen die zeiten und cuenumbers rein
				if (FSetTimes[0] == true) {
					
					for (int i = 0; i < FData.Rows.Count; i++) FDataGridView.Rows[i].Cells[0].Value=FCueNumbers[i];
					for (int i = 0; i < FData.Rows.Count; i++) FDataGridView.Rows[i].Cells[1].Value=FTimeCodes[i];
				
			    }
				
				
				
				
				
				if (FDataGridView.SelectedCells.Count > 0)
					selectedRow = FDataGridView.SelectedCells[0].RowIndex;
				else
					selectedRow = 0;
				
				
				// hier bewegen wir die Markierung und scrollen automatisch FSetIndex.IsChanged
				if (FGo[0]== true)
				{
				selectedRow = FSetIndex[0];
				moveRow = true;
				FDataGridView.FirstDisplayedScrollingRowIndex = FSetIndex[0];
					
				
				}
				
				
				
				

				if (moveRow)
				{
					FDataGridView.ClearSelection();
					FDataGridView.Rows[selectedRow].Selected = true;
				}
				
			}

			if (FNeedsUpdate)
			{
				FDataGridView.Refresh();
				FNeedsUpdate = false;
				
			}

			if (FData.Rows.Count == 0)
			{
				FCurrentIndex.SliceCount = 1;
				FCurrentIndex[0] = 0;
			}
			else
			{
				
				var rows = FDataGridView.SelectedRows;
				if (rows.Count > 0)
				{
					
					FCurrentIndex.SliceCount = 0;
					foreach (DataGridViewRow row in rows)
					{
						FCurrentIndex.Add(row.Index);
					}
				}
				else
				{
					int row = FDataGridView.CurrentCellAddress.Y;
					FCurrentIndex.SliceCount = 1;
					FCurrentIndex[0] = row;
					
				}
			}
		}

		void FData_DataChanged(Object sender, EventArgs e)
		{
			//pretty hacky. this clears the 'udpate' flag if the last instruction is from itself
			FNeedsUpdate = true;// (sender != this);
			
		}

		private void ValueTableBufferNode_Resize(object sender, EventArgs e)
		{
			this.FDataGridView.Size = this.Size;
		}

		private void dataGridView1_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
		{
			try
			{
				System.Convert.ToString(e.FormattedValue);
				
				
			}
			catch
			{
				e.Cancel = true;
				
			}			
			ReformatTable();
		}

		private void ReformatTable()
		{
			
		}

	}
}
